// File Name : main.c
// Programmed by Jimmy Seo
// Program Date : Mon., May 10, 2004
//
#include <GC80C520_PL44I.H>
#include "timer.h"

// Declaration of the Variables
xdata unsigned char *pcode;				  // for External Memory
data unsigned char  temp1;				  // for Temporary Data
data unsigned char  i;					  // for Loop Parameters
data unsigned char  p;					  // for Loop Parameters
data unsigned char	port_change;		  // for PORT Display Delection
data unsigned char	port_value_change;	  // for PORT Value Update Change
data unsigned char	port_display_change;  // for PORT Display Selection

int loop_cnt;	// for Loop Parameter

// Prototype Functions
// 1. System Initialization
void initialize(void);

// 2. Led Output
void output_led(void);

// 3. Segment Output
void output_segment(void);

// 4. delay fuction
void delay (int);


// Main Function
//
void main(void) {

	initialize();

	
	loop_cnt = 0;

	TR0 = 1;	// Timer0 Start (Enable)

	
	while(1) {
		
		// Description for Address Mapping
			
		// pcode = 0xFEXX : PORTB (and 7-segment Display)
		// pcode = 0xFDXX : PORTC (and 8 bits LED Display)
	
		switch (port_display_change) {
			case 0 :	// PORTC
				output_led();
				
				if (port_value_change==1) {
			  		port_display_change=1;
					port_value_change = 0;
				}
				break;
			
			case 1 :	// PORTB
				output_segment();
			
				if (port_value_change==1){
					port_display_change=0;
					port_value_change = 0;
				}	
				break;
			
		} // End: switch ()

	} // End: while(1)
	
} // End: void Main(void)



// LED Display
void output_led(void){
	
	//pcode = (unsigned char xdata *)0xFD00;
	pcode = 0xF500;
	
	temp1 =0x01;

	for (p=0; p<8; p++) {
		delay(30);				
		temp1 = (temp1<<1);
		*(pcode) = temp1;
	}

	temp1 = 0x80;

	for (p=0; p<8; p++) {
		delay(30);				
		temp1 = (temp1>>1);
		*(pcode) = temp1;
	} // End: for (p=0; p<8; p++)
	
			
}
		


// 7-segment Display
void output_segment(void){
	
	//pcode = (unsigned char xdata *)0xFE00;
	pcode = 0xF600;
		
	*(pcode) = 0x3F;	// Digit : 0
	delay(100);
	
	*(pcode) = 0x06;	// Digit : 1
	delay(100);
	
	*(pcode) = 0x5B;	// Digit : 2
	delay(100);
	
	*(pcode) = 0x4F;	// Digit : 3
	delay(100);
	
	*(pcode) = 0x66;	// Digit : 4
	delay(100);
	
	*(pcode) = 0x6D;	// Digit : 5
	delay(100);
	
	*(pcode) = 0x7D;	// Digit : 6
	delay(100);
	
	*(pcode) = 0x27;	// Digit : 7
	delay(100);
	
	*(pcode) = 0x7F;	// Digit : 8
	delay(100);
	
	*(pcode) = 0x6F;	// Digit : 9
	delay(100);
	
	*(pcode) = 0x3F;	// Digit : 0
	delay(100);
}


	
// Sub Function - 1. void initialize(void)
void initialize(void) {
	TMOD  = 0x22; 	// Timer 0/1 : Mode 2<Auto Reload>
					
    
	TH0   = 0x06;   // TH0 for Auto-reload
					// (Fosc/12)^-1*(256-TH0) = Timing Duty
					// When Fosc = 12MHz and TH0 = 0x06,
					// Timing Duty is 0.25msec (250 usec)
		
	pcode = (unsigned char xdata *)0x0000;

	ET0 = 1;		// Timer 0 Interrupt Enable
	EA =  1;			// All Interrupts Enable

} // End: void initialize(void)


/* delay function for m seconds */
void delay(int Delay)
{
	int i, j;
	for(j=0; j<Delay; j++)
		for (i=0; i<100; i++);
}


// Sub Function : Timer 0 Interrupt
void timer0_int(void) interrupt TF0_VECTOR {
	loop_cnt++;
	
	if (loop_cnt == 4000) {	 // 4000 : 1sec, 1000 : 0.25 sec, 500 : 0.125 sec)
		loop_cnt = 0;
			
		if (port_change > 10) {
			port_value_change = 1;
			port_change = 0;
		}
		port_change++;
		
	}
	TF0 = 0;					 // Timer 0 Overflow Falg Clear

} // End : Sub Function - void timer0_int(void) interrupt TF0_VECTOR
