#include <GC80C520_PL44I.H>

xdata unsigned char *pcode;				    // for External Memory

//remote V
data unsigned char start_value;
data unsigned char rbuf;
data unsigned long tvcode;
data unsigned long stvcode;
data unsigned char dc;
data unsigned char temp;
 
//ex F
void initialize(void);
void read_code(void);
void check_code(unsigned char);
void output_segment(char);
void delay(int);

void main(void) {

	initialize();
		
	while(1);


}



// Sub Function : initialize
void initialize(void) {
		
    TH0 = 0;
    TL0 = 0;
    
    TMOD  = 0x11; 	// Timer 0/1 : Mode 1 <16bit-timer>
					
    
	EX0 = 1;		// Ex 0 interrupt enable
	IT0 = 1;		// Ex 0 edge-mode

	ET0 = 1;		// Timer 0 interrupt enable
	
	EA =  1;		// All Interrupts Enable
	
	TR0 = 1;
	
}


// Sub Function : External 0 Interrupt
void ie0_int(void) interrupt IE0_VECTOR {
	TR0 = 0;
	read_code();
	TR0 = 1;
	IE0 = 0;
}


// Sub Function : Timer 0 Interrupt
void timer0_int(void) interrupt TF0_VECTOR {
	start_value = 0;
	TF0 = 0;
}


// Sub Function : read code
void read_code(void) {
	char h, l;
	long t;
	
	h = TH0;
	l = TL0;
	TH0 = 0;
	TL0 = 0;
	
	t = h * 256 + l;				
	
	
	if (start_value == 0) {
		stvcode = 0;
		tvcode = 0;
		start_value = 1;
	}
	
	else if (start_value == 1) {
		if ( 7000 < t && t < 10000 ) {
			start_value = 2;
			rbuf = 0;
			dc = 0;
		}
		else start_value = 0;
	}
	else if (start_value == 2) {
		if ( 300 < t && t < 1300 ) {
			dc++;
			rbuf *= 2;
			if ( dc > 8 ) {
				start_value = 0;
			}
		}
		else if ( 1300 < t && t < 2500 ) {
			dc++;
			rbuf *= 2;
			rbuf += 1;
			if ( dc > 8 ) {
				start_value = 0;
			}
		}
		else start_value = 0;
		if ( dc == 8 ) { 
			check_code(rbuf);
			dc = 0;
			rbuf = 0;
		}
				
	}
	else start_value = 0;
	
} // End: void read_code(void)


// Sub Function : Timer 0 Interrupt
void check_code(unsigned char buff) {

	if (stvcode == 0) {
		if (tvcode == 0) tvcode = buff;
		else if (tvcode == 0xe0 && buff == 0xe0) stvcode = 1;
		else start_value = 0;
	}
	
	else if (stvcode == 1) {
	
		if (buff == 0x20) { 
			start_value = 0;
			output_segment(1);
		}
		
		else if (buff == 0xa0) {
			start_value = 0;
			output_segment(2);
		}
		
		else if (buff == 0x60) {
			start_value = 0;
			output_segment(3);
		}
		
		else if (buff == 0x10) {
			start_value = 0;
			output_segment(4);
		}
		
		else if (buff == 0x90) {
			start_value = 0;
			output_segment(5);
		}
		
		else if (buff == 0x50) {
			start_value = 0;
			output_segment(6);
		}
		
		else if (buff == 0x30) {
			start_value = 0;
			output_segment(7);
		}
		
		else if (buff == 0xb0) {
			start_value = 0;
			output_segment(8);
		}
		
		else if (buff == 0x70) {
			start_value = 0;
			output_segment(9);
		}
		
		else if (buff == 0x88) {
			start_value = 0;
			output_segment(0);
		}
		
		else if (buff == 0x40) {	// power button
			start_value = 0;
			output_segment(40);
		}
		
		else start_value = 0;
		
	}
} // End: void check_code(unsigned char)


// 7-segment Display
void output_segment(char c){
	
	pcode = (unsigned char xdata *)0xF600;
		
	if(c==0){
		*(pcode) = 0x3F;	// Digit : 0
		delay(100);
	}
	
	else if(c==1){
		*(pcode) = 0x06;	// Digit : 1
		delay(100);
	}
	
	else if(c==2){
		*(pcode) = 0x5B;	// Digit : 2
		delay(100);
	}
	
	else if(c==3){
		*(pcode) = 0x4F;	// Digit : 3
		delay(100);
	}
	
	else if(c==4){
		*(pcode) = 0x66;	// Digit : 4
		delay(100);
	}
	
	else if(c==5){
		*(pcode) = 0x6D;	// Digit : 5
		delay(100);
	}
	
	else if(c==6){
		*(pcode) = 0x7D;	// Digit : 6
		delay(100);
	}
	
	else if(c==7){
		*(pcode) = 0x27;	// Digit : 7
		delay(100);
	}
	
	else if(c==8){
		*(pcode) = 0x7F;	// Digit : 8
		delay(100);
	}
	
	else if(c==9){
		*(pcode) = 0x6F;	// Digit : 9
		delay(100);
	}
	
	else if(c==0){
		*(pcode) = 0x3F;	// Digit : 0
		delay(100);
	}
	
	else if(c==40){
		*(pcode) = 0x00;	// Digit : clear
		delay(100);
	}
	
} // End: void output_segment(char)


// delay function
void delay(int a)
{	             
	int i;
	for(i = 0; i < a; i++);
} // End : void out
